Extension { #name : 'GsProcess' }

{ #category : 'Accessing' }
GsProcess class >> methodAt: aLevel [
  "Return the GsNMethod in which the sender of methodAt: is executing."
  ^ GsProcess _methodInFrameContents:(GsProcess _frameContentsAt: aLevel + 1) 

]

{ #category : 'Private' }
GsProcess >> _finishTermination [
  "returns receiver"
  | wasInCrit sched |
  wasInCrit := (sched := self _scheduler) _enterCritical .
  block ifNil: [
    uaCount_depth := 0.
    arStack := nil.
    interruptFlag_stEntryCount := 0 .
  ] ifNotNil:[
    block := nil
  ].
  "remoteProcess := nil. "
  self _setModeinfoBit: ModeInfo_terminated value: 1 .
  self _signalJoiners .
  wasInCrit == 0 ifTrue:[ sched _exitCritical ].
  sched _terminateScheduledProcess: self.
]

{ #category : 'Private' }
GsProcess >> _serviceTerminationInterrupt [
  [
    self _executeEnsureBlocks .
  ] on: Error do:[:ex |
    "GsFile gciLogServer: '-- Failure during ensure blocks, ' , ex asString ."
    ex return: nil ."ignore exception"
  ].
  [
    ^ self _finishTermination
  ] on: Error do:[:ex |
    GsFile gciLogServer: ('-- Failure during _finishTermination, ' , ex asString) lf ;
          gciLogServer: ex stackReport .
    ex return: nil ."ignore exception"
  ]

]

{ #category : 'Private' }
GsProcess >> _terminatedReturnToGci: anException [

"If receiver was not the most recently started main GsProcess,
 i.e. not from most recent IntSendMsg or IntExecute ,
 return the receiver.

 Otherwise return control to the GCI with a GciErrorSType encapsulating
 anException .  The receiver will be the GsProcess associated with that
 GciErrorSType , and will have no stack."

<primitive: 543>
self _primitiveFailed: #_terminatedReturnToGci args: { anException } .
self _uncontinuableError  "should never reach here"

]

{ #category : 'Changing Process State' }
GsProcess >> terminate [
 "Terminates the receiver. If receiver is not the current process,
  it will be raised to normal priority and sent a termination interrupt.

  If receiver is a forked process, i.e. not the GsProcess of the
  current execute or perform, returns the receiver after receiver
  has been terminated.

  If receiver is a non-forked process , then
  after completion of the termination,  a not-trappable
  an error is returned to the GCI, containing
  a not-trappable instance of TerminateProcess,
  The   instance of TerminateProcess may have a stack captured,
  but   the GsProcess associated with the error has no stack
  and is terminated."

  ^ self terminateTimeoutMs: 30000"wait up to 30 seconds for ensure blocks to complete"

]

{ #category : 'Changing Process State' }
GsProcess >> terminate9 [
  "agressive termination that executes in the sender's green thread
   without yielding and without using the receiver's stack to execute
   the ensure blocks ."
  | minfo |
  ((minfo := modeInfo) bitAnd: ModeInfo_continuationMask) ~~ 0 ifTrue:[
    self error:'cannot terminate an instance that is a continuation'
  ].
  (minfo bitAnd: ModeInfo_terminationStarted) == 0 ifTrue:[
    "ensure blocks not attempted yet"
    [
      self _executeEnsureBlocks .
    ] on: Error do:[ :ex |
      "GsFile gciLogServer: '-- Failure during ensure blocks, ' , ex asString ."
      ex return: nil  "abandon further execution of ensure blocks"
    ]
  ].
  self _finishTermination .

]

{ #category : 'Changing Process State' }
GsProcess >> terminateTimeoutMs: timeoutMs [
 "Terminates the receiver. If receiver is not the current process,
  it will be raised to normal priority and sent a termination interrupt.

  The argument timeoutMs is a SmallInteger specifying maximum time
  in milliseconds to wait for ensure blocks of the receiver to finish.
  If termination is not complete after this time, an Error is signalled.

  If receiver is a forked process, i.e. not the GsProcess of the
  current execute or perform, returns the receiver after receiver
  has been terminated.

  If receiver is a non-forked process , then after completion of the termination,  
  a not-trappable error is returned to the GCI, containing
  a not-trappable instance of TerminateProcess,
  The   instance of TerminateProcess may have a stack captured,
  but   the GsProcess associated with the error has no stack
  and is terminated .
  "
  | minfo theEx |
  ((minfo := modeInfo) bitAnd: ModeInfo_continuationMask) ~~ 0 ifTrue:[
     ImproperOperation new object: self;
           signal:'cannot terminate an instance that is a continuation' .
  ].
  [  Notification signal ] onSynchronous: AbstractException do:[ :ex |
    "capture stack in case GemExceptionSignalCapturesStack is true"
    (theEx := TerminateProcess new) _gsStack: ex _gsStack .
  ].
  (minfo bitAnd: ModeInfo_terminationStarted) ~~ 0 ifTrue:[ "termination already started"
      "GsFile gciLogServer:'-- termination already started'."
      self _finishTermination .
  ] ifFalse:[ | currentProc |
      self _setModeinfoBit: ModeInfo_terminationStarted value: 1 .
      (self == (currentProc := self _current)) ifTrue:[ "receiver is the active process"
        "GsFile gciLogServer:'-- _serviceTerminationInterrupt' ."
        self _serviceTerminationInterrupt .
      ] ifFalse:[ | sched cPrio slept |
        "GsFile gciLogServer:'-- beforked ' ."
        self beForked . 
        (sched := self _scheduler) _enterCritical .
        self _storeBit: 3 value: true . "set INT_GSPROC_TERMINATE_MASK bit"
        self _unscheduleProcess . "undo waiting on delays, semaphores, etc"
        self priority <= (cPrio := currentProc priority) ifTrue:[
          self _setPriority: cPrio + 1 .
        ].
        sched _scheduleProcess: self "put on readyQ";
              _exitCritical .
        slept := 0 .
        [ self _isTerminated ] whileFalse:[
          slept > timeoutMs ifTrue:[ ThreadError signal: 'Termination waited too long'].
          Delay waitForMilliseconds: 10.
          slept := slept + 10 .
        ].
      ]
  ] .
  "self _isTerminated ifFalse:[
    GsFile gciLogServer:'-- termination not complete, skip terminate9'.
    self terminate9
  ]."
  (minfo bitAnd: ModeInfo_forked) == 0 ifTrue:[
    "GsFile gciLogServer:'-- terminatedReturnToGci'."
    self _isTerminated ifFalse:[
      Error signal:'Termination failed to complete.'
    ].
    self _terminatedReturnToGci: theEx
  ].

]
